import fetch from 'node-fetch';
import fs from 'fs/promises';
import path from 'path';

interface AssistantSettings {
  silence_timeout: number;
  assistant_name: string;
  first_message: string;
  system_prompt: string;
  model_provider: string;
  model: string;
  temperature: number;
  top_p: number;
  max_tokens: number;
  transcriber_provider: string;
  transcriber_model: string;
  transcriber_language: string;
  voice_provider: string;
  voice_id: string;
  stability: number;
  clarity_boost: number;
  phone_number: string;
  forwarding_phone_number: string;
  end_call_phrases: string;
  words_to_interrupt: number;
  clear_message_delay: number;
  maximum_duration: number;
  response_delay: number;
  llm_request_delay: number;
  server_url: string;
  voicemail_message: string;
  end_call_message: string;
  add_voice_id_manually: boolean;
  enable_end_call_function: boolean;
  dial_keypad: boolean;
  enable_interruptions: boolean;
  hipaa_compliance: boolean;
  recording_enabled: boolean;
}

let cachedSettings: AssistantSettings | null = null;

export const config = {
  voice360ApiKey: process.env.VOICE360_API_KEY || '',
  assistantId: process.env.ASSISTANT_ID || '',
  perfexCrmUrl: process.env.PERFEX_CRM_URL || '',
};

const defaultSettings: AssistantSettings = {
  silence_timeout: 30,
  assistant_name: '',
  first_message: '',
  system_prompt: '',
  model_provider: 'openai',
  model: 'gpt-4o',
  temperature: 0.7,
  top_p: 1,
  max_tokens: 250,
  transcriber_provider: 'deepgram',
  transcriber_model: 'nova',
  transcriber_language: 'us',
  voice_provider: 'elevenlabs',
  voice_id: '',
  stability: 0.5,
  clarity_boost: 0.75,
  phone_number: '',
  forwarding_phone_number: '',
  end_call_phrases: '',
  words_to_interrupt: 2,
  clear_message_delay: 0,
  maximum_duration: 1800,
  response_delay: 0.1,
  llm_request_delay: 0.1,
  server_url: '',
  voicemail_message: '',
  end_call_message: '',
  add_voice_id_manually: false,
  enable_end_call_function: false,
  dial_keypad: false,
  enable_interruptions: false,
  hipaa_compliance: false,
  recording_enabled: false,
};

export async function fetchAssistantSettings(assistantId: string): Promise<AssistantSettings> {
  try {
    const url = new URL(`/admin/voice360/API/get_assistant_settings/${assistantId}`, config.perfexCrmUrl);
    
    console.log('Fetching assistant settings from:', url.toString());
    console.log('Using API Key:', config.voice360ApiKey);

    const response = await fetch(url.toString(), {
      headers: {
        'Accept': 'application/json',
        'X-Requested-With': 'XMLHttpRequest',
        'Authorization': `Bearer ${config.voice360ApiKey}`
      }
    });

    if (!response.ok) {
      const errorText = await response.text();
      console.error('Error response body:', errorText);
      throw new Error(`Failed to fetch assistant settings. Status: ${response.status}, Error: ${errorText}`);
    }

    const data = await response.json();
    console.log('Fetched assistant settings:', JSON.stringify(data, null, 2));

    if (!data || typeof data !== 'object' || Object.keys(data).length === 0) {
      console.error('Invalid or empty data received from the API');
      return defaultSettings;
    }

    console.log('Assistant ID:', assistantId);
    console.log('Received data type:', typeof data);
    console.log('Received data keys:', Object.keys(data));

    const settings: AssistantSettings = {
      ...defaultSettings,
      ...data,
      silence_timeout: parseInt(data.silence_timeout) || defaultSettings.silence_timeout,
      temperature: parseFloat(data.temperature) || defaultSettings.temperature,
      top_p: parseFloat(data.top_p) || defaultSettings.top_p,
      max_tokens: parseInt(data.max_tokens) || defaultSettings.max_tokens,
      stability: parseFloat(data.stability) || defaultSettings.stability,
      clarity_boost: parseFloat(data.clarity_boost) || defaultSettings.clarity_boost,
      words_to_interrupt: parseInt(data.words_to_interrupt) || defaultSettings.words_to_interrupt,
      clear_message_delay: parseInt(data.clear_message_delay) || defaultSettings.clear_message_delay,
      maximum_duration: parseInt(data.maximum_duration) || defaultSettings.maximum_duration,
      response_delay: parseFloat(data.response_delay) || defaultSettings.response_delay,
      llm_request_delay: parseFloat(data.llm_request_delay) || defaultSettings.llm_request_delay,
      add_voice_id_manually: data.add_voice_id_manually === '1',
      enable_end_call_function: data.enable_end_call_function === '1',
      dial_keypad: data.dial_keypad === '1',
      enable_interruptions: data.enable_interruptions === '1',
      hipaa_compliance: data.hipaa_compliance === '1',
      recording_enabled: data.recording_enabled === '1',
    };

    // No need to remove nested settings as they don't exist in the interface

    // Update the assistantSettings.json file
    const cachePath = path.join(__dirname, '..', '..', 'cache', 'assistantSettings.json');
    let currentSettings: AssistantSettings[];
    try {
      await fs.access(cachePath);
      const fileContent = await fs.readFile(cachePath, 'utf8');
      currentSettings = fileContent ? JSON.parse(fileContent) : [];
    } catch (error) {
      if ((error as NodeJS.ErrnoException).code === 'ENOENT') {
        console.log('assistantSettings.json does not exist, creating it');
        currentSettings = [];
        await fs.mkdir(path.dirname(cachePath), { recursive: true });
        await fs.writeFile(cachePath, JSON.stringify(currentSettings, null, 2));
      } else {
        console.error('Error reading assistantSettings.json:', error);
        currentSettings = [];
      }
    }

    // Find the existing setting for this assistant
    const existingSettingIndex = currentSettings.findIndex(setting => setting.assistant_name === settings.assistant_name);
    
    if (existingSettingIndex !== -1) {
      // Update existing setting
      currentSettings[existingSettingIndex] = settings;
    } else {
      // Add new setting
      currentSettings.push(settings);
    }

    // Write updated settings back to file
    try {
      await fs.writeFile(cachePath, JSON.stringify(currentSettings, null, 2));
      console.log('Updated assistantSettings.json:', JSON.stringify(currentSettings, null, 2));

      // Verify the file contents after writing
      const verifyContent = await fs.readFile(cachePath, 'utf8');
      console.log('Verified contents of assistantSettings.json:', verifyContent);
    } catch (error) {
      console.error('Error writing to assistantSettings.json:', error);
    }

    cachedSettings = settings;
    console.log('Assistant settings fetched successfully', { silenceTimeoutSeconds: settings.silence_timeout });

    // Ensure the file is not empty by reading it back
    try {
      const verifyContent = await fs.readFile(cachePath, 'utf8');
      if (!verifyContent || verifyContent.trim() === '') {
        console.error('assistantSettings.json is empty after write operation');
        // Attempt to write the content again
        await fs.writeFile(cachePath, JSON.stringify(currentSettings, null, 2));
        console.log('Attempted to rewrite assistantSettings.json');
      } else {
        console.log('assistantSettings.json successfully updated and contains data');
        // Log the contents of assistantSettings.json
        console.log('Contents of assistantSettings.json:', JSON.stringify(currentSettings, null, 2));
      }
    } catch (error) {
      console.error('Error verifying assistantSettings.json content:', error);
    }

    return settings;
  } catch (error) {
    console.error('Error fetching assistant settings:', error);
    cachedSettings = defaultSettings;
    return defaultSettings;
  }
}

export function updateCachedSettings(newSettings: AssistantSettings) {
  cachedSettings = newSettings;
}

export function getCachedSettings(): AssistantSettings | null {
  return cachedSettings;
}

export default config;
